@@ -5,6 +5,7 @@ import re
 import textwrap
 import warnings
 from contextlib import suppress
+from dataclasses import dataclass
 from functools import partial
 from types import MappingProxyType
 from typing import (
@@ -30,7 +31,14 @@ from typing import (
 import docstring_parser as docs
 from typing_extensions import Annotated
 
-from hiddenv.environment import Env, NotNull, as_property_name, empty
+from hiddenv.environment import (
+    Env,
+    NotAnnotated,
+    NotNull,
+    as_property_name,
+    empty,
+    validate_setting_value,
+)
 
 
 _Tco = TypeVar("_Tco", covariant=True)
@@ -43,15 +51,11 @@ class KwargsCallable(Protocol[_Tco]):  # pylint: disable = too-few-public-method
         ...
 
 
-class SettingInfo(NamedTuple):
-    """Metadata for a Config subclass' annotated setting."""
-
+@dataclass(frozen=True)
+class _Info:
     key: str
     annotation: Any
-    description: str
-    parent: Type["Config"]
     owner: Type["Config"]
-    default: Any = empty
 
     @property
     def name(self):
@@ -71,6 +75,69 @@ class SettingInfo(NamedTuple):
 
         return isinstance(getattr(self.owner, self.property_name, None), property)
 
+    @property
+    def is_immutable(self):
+        """Whether annotation is immutable (e.g. ClassVar) or not."""
+
+        annotation = get_origin(self.annotation) or self.annotation
+        return annotation in (get_origin(ClassVar[str]), get_origin(Final[str]))
+
+
+@dataclass(frozen=True)
+class _AnnotationInfo(_Info):
+    source: type
+
+
+@dataclass(frozen=True)
+class SettingInfo(_Info):
+    """Metadata for a Config subclass' annotated setting."""
+
+    parent: Type["Config"]
+    description: str
+    default: Any = empty
+
+    @property
+    def allow_null(self):
+        """Whether None is allowed as a value."""
+
+        return self.default is None or self.is_property
+
+    @property
+    def attr_ann(self):
+        """Attribute annotation for this setting."""
+
+        assert not self.is_immutable
+        ann = NotAnnotated[self.annotation]
+        if self.allow_null:
+            ann = NotNull[self.annotation]
+        assert get_origin(ann) is not Union, (self.key, self.annotation, ann)
+        if self.is_property:
+            assert isinstance(ann, type) or get_origin(ann) in (list, tuple, dict), (self.key, self.annotation, ann)
+            return ann
+        assert hasattr((get_origin(ann) or ann), "__name__"), (self.key, self.annotation, ann)
+        env_call = getattr(Env, (get_origin(ann) or ann).__name__, None)
+        process_name = f"_process_{self.property_name}"
+        process_call = getattr(self.owner, process_name, None)
+        if callable(env_call) or callable(process_call):
+            return ann
+        if isinstance(ann, type) and issubclass(ann, Config):
+            return ann
+        assert False, f"missing {process_name} for {self.key}: {inspect.formatannotation(self.annotation)}"
+
+    @property
+    def is_config(self):
+        """Whether the attribute annotation is a Config subclass."""
+
+        return isinstance(self.attr_ann, type) and issubclass(self.attr_ann, Config)
+
+    @property
+    def env_name(self):
+        """Environment variable name."""
+
+        if isinstance(self.annotation, type(Annotated[str, ""])) and isinstance(self.annotation.__metadata__[-1], str):
+            return self.annotation.__metadata__[-1]
+        return self.name
+
     @property
     def dotenv(self):
         """Textual .env representation of this setting."""
@@ -78,10 +145,7 @@ class SettingInfo(NamedTuple):
         if self.is_property:  # coverage: exclude
             raise ValueError(f"{_Reserved(self.owner)}.{self.key} is a property setting")
 
-        name = self.name
-        if isinstance(self.annotation, type(Annotated[str, ""])):
-            if len(self.annotation.__metadata__) == 1 and isinstance(self.annotation.__metadata__[0], str):
-                name = self.annotation.__metadata__[0]
+        name = self.env_name
         if self.owner.namespace:
             name = f"{self.owner.namespace}_{name}"
         text = f"{name}="
@@ -98,6 +162,9 @@ class SettingInfo(NamedTuple):
             text = f"# setting {name}{default_text}\n{text}"
         return f"\n{text}"
 
+    def __post_init__(self):
+        assert self.is_immutable or self.attr_ann
+
     def __str__(self):
         text = f"{self.key}: {inspect.formatannotation(self.annotation)}"
         if self.default is not empty:
@@ -111,10 +178,6 @@ class SettingInfo(NamedTuple):
 EMPTY_NAMESPACE: Final[str] = type("CustomNamespace", (str, ), dict(__slots__=()))()
 
 
-def _is_immutable(annotation):
-    return annotation is Final or get_origin(annotation) in (get_origin(ClassVar[str]), get_origin(Final[str]))
-
-
 class _Reserved(NamedTuple):
     source: type
 
@@ -126,7 +189,7 @@ class _Reserved(NamedTuple):
             if "__reserved__" in self.source.__dict__:
                 return tuple(self.source.__reserved__)
             return tuple(
-                k for k in {*self.source.__dict__}.difference(self.source.__annotations__)
+                k for k in {*self.source.__dict__}.difference(getattr(self.source, "__annotations__", {}))
                 if not isinstance(getattr(self.source, as_property_name(k)), property)
             )
         return tuple(dir(self.source))  # coverage: exclude
@@ -144,20 +207,6 @@ class _Reserved(NamedTuple):
         return f"{self.source.__module__}.{self.source.__qualname__}"
 
 
-class _Annotation(NamedTuple):
-    key: str
-    annotation: Any
-    source: type
-    name: str
-    property_name: str
-
-    @property
-    def immutable(self):
-        """Whether annotation is immutable (e.g. ClassVar) or not."""
-
-        return _is_immutable(self.annotation)
-
-
 _T = TypeVar("_T")
 
 
@@ -173,39 +222,38 @@ def load(__call: KwargsCallable[_T], __get: Callable[[str], Any], **overrides) -
     return __call(**overrides)
 
 
-def _get_doc(target, description, **attribute_descriptions):
-    if isinstance(getattr(target, "__doc__", None), str) and getattr(target, "__doc__").strip():
-        parsed = docs.parse(getattr(target, "__doc__"))
-        if description is None:
-            description = (parsed.short_description or "").strip()
-            if parsed.long_description:
-                description = f"{description}\n\n{parsed.long_description.strip()}"
+def _get_doc(cls, target, descriptions):
+    if descriptions is not None:
+        descriptions = dict(descriptions)
+        descriptions.setdefault(None, "")
+    else:
+        descriptions = {}
+        parsed = docs.parse(getattr(target, "__doc__", None) or "")
+        descriptions[None] = (parsed.short_description or "").strip()
+        if parsed.long_description:  # coverage: exclude
+            descriptions[None] = f"{descriptions[None]}\n\n{parsed.long_description.strip()}"
         for param in parsed.params:
-            attribute_descriptions.setdefault(param.arg_name, param.description)  # type: ignore
-
-    parts = [f"{textwrap.dedent((description or '').rstrip())}\n"]
-    if attribute_descriptions:
+            descriptions[cls.as_setting_name(param.arg_name)] = param.description or ""
+    parts = [f"{textwrap.dedent(descriptions.pop(None).rstrip())}\n"]
+    if descriptions:
         parts.append("Attributes:")
-    for k, v in attribute_descriptions.items():
+    for k, v in descriptions.items():
         parts.append(f"    {k}:\n{textwrap.indent(textwrap.dedent(v).rstrip(), ' ' * 8)}")
     return "\n".join(parts).rstrip()
 
 
-def _get_class_namespace(cls, target, skip_parameters, raise_error) -> dict:
-    if target is None:
-        return {}
+def _get_attributes_from_target(cls, target, exclude, raise_error) -> Dict[str, Union[Type, Tuple[Type, Any]]]:
 
     skip = f"while extending {cls.__module__}.{cls.__qualname__} for {target!r}, skipping parameter {{}}: {{}}"
 
     existing: Dict[str, Any] = {cls.as_setting_name(k): ann for k, ann in cls.__annotations__.items()}
 
-    annotations: Dict[str, Any] = {}
-    defaults: Dict[str, Any] = {}
+    attributes: Dict[str, Union[Type, Tuple[Type, Any]]] = {}
 
-    for i, parameter in enumerate(inspect.signature(target).parameters.values()):
-
-        if i in skip_parameters or parameter.name in skip_parameters:
-            continue
+    for parameter in (
+        parameter for i, parameter in enumerate(inspect.signature(target).parameters.values())
+        if i not in exclude and parameter.name not in exclude
+    ):
 
         if parameter.kind in (inspect.Parameter.VAR_KEYWORD, inspect.Parameter.VAR_POSITIONAL):
             reason = skip.format(parameter.name, "variadic parameters not allowed")
@@ -222,6 +270,9 @@ def _get_class_namespace(cls, target, skip_parameters, raise_error) -> dict:
             continue
 
         name = cls.as_setting_name(parameter.name)
+        if name in exclude:
+            continue
+
         if not cls.is_setting(name):
             reason = skip.format(parameter.name, f"the generated setting name {name} is invalid")
             if raise_error:
@@ -240,30 +291,35 @@ def _get_class_namespace(cls, target, skip_parameters, raise_error) -> dict:
                 ),
             )
 
-        if name in annotations:
+        if name in attributes:
             raise TypeError(skip.format(parameter.name, f"duplicate setting name {name}"))
 
-        annotations[name] = parameter.annotation
-
         if parameter.default is not inspect.Parameter.empty:
-            defaults[name] = parameter.default
+            attributes[name] = (parameter.annotation, parameter.default)
+        else:
+            attributes[name] = parameter.annotation
 
-    return {
-        "__annotations__": annotations,
-        **defaults,
-    }
+    return attributes
 
 
 class Config:
     """Settings class for use in specific cases (as opposed to project-wide) or as part of a source settings module.
 
-    Attributes:
+    Attributes:  # noqa
         dotenv:
             Textual .env file representation of settings for this Config class, generated using `.__info__`
         name_pattern:
             regex pattern for valid setting (or namespace) name
         namespace:
             prefix to use when getting settings from environment variables
+
+    Args:
+        namespace:
+            prefix to use when getting settings from environment variables;
+            defaults to class namespace
+        __env:
+        __app: App name, primarily for django-related Config subclass usage.
+        **attributes: additional annotations to use
     """
 
     __info__: ClassVar[Mapping[str, SettingInfo]] = MappingProxyType({})
@@ -273,35 +329,64 @@ class Config:
     name_pattern: ClassVar[Pattern[str]] = re.compile(r"^[A-Z][A-Z0-9]*(__?[A-Z0-9]+)*$")
 
     @classmethod
-    def extend_for(
+    def extend(
         cls: "_TConfig",
-        *,
         namespace: str = "",
-        target: Optional[Callable] = None,
-        skip_parameters: Collection[Union[int, str]] = (),
-        raise_error=False,
-        description: str = None,
-        **attribute_descriptions: str,
+        exclude: Collection[str] = (),
+        doc: str = "",
+        **attributes: Union[Type, Tuple[Type, Any]],
     ) -> "_TConfig":
-        """Returns a subclass update using given input.
+        """Returns a subclass of this Config class.
 
         Args:
-            namespace: new namespace for the class
-            target: callable to get additional setting parameters from
-            skip_parameters: names of target parameters to skip
-            raise_error: whether to raise an error on invalid target parameters
-            description: new class description; do not use with doc parameter
-            **attribute_descriptions: new settings' descriptions; do not use with doc parameter
+            doc: docstring/descriptions of new subclass
+            exclude: names of existing setting attributes to exclude from new subclass
+            namespace: namespace for the new subclass
+            **attributes: new setting attributes, as annotation or (annotation, default) tuples
         """
 
-        class_namespace = _get_class_namespace(cls, target, skip_parameters, raise_error)
-        class_namespace["__doc__"] = _get_doc(target, description, **attribute_descriptions)
+        defaults = {}
+        for k, v in attributes.items():
+            if isinstance(v, tuple):
+                attributes[k], defaults[k] = v  # pylint: disable = unnecessary-dict-index-lookup
 
         return type(  # type: ignore
             cls.__name__,
             (cls, ),
-            class_namespace,
+            {
+                "__doc__": doc,
+                "__annotations__": attributes,
+                **defaults,
+            },
             namespace=namespace,
+            exclude=exclude,
+        )
+
+    @classmethod
+    def extend_for(
+        cls: "_TConfig",
+        target: Callable,
+        *,
+        exclude: Collection[Union[int, str]] = (),
+        namespace: str = "",
+        descriptions: Dict[Optional[str], str] = None,
+        raise_error=False,
+    ) -> "_TConfig":
+        """Returns a subclass update with setting attributes from given target's call signature.
+
+        Args:
+            target: callable to get additional setting parameters from
+            exclude: positions or names of target parameters; and/or names of existing setting attributes to exclude
+            namespace: namespace for the new subclass
+            descriptions: new settings' descriptions; use None key for the new class description
+            raise_error: whether to raise an error on invalid target parameters
+        """
+
+        return cls.extend(
+            namespace=namespace,
+            exclude=tuple(key for key in exclude if isinstance(key, str)),
+            doc=_get_doc(cls, target, descriptions),
+            **_get_attributes_from_target(cls, target, exclude, raise_error),
         )
 
     @property
@@ -318,25 +403,34 @@ class Config:
 
         return self.__app
 
-    def __init__(
-        self,
+    def __new__(
+        cls,
         namespace: Optional[str] = None,
         __env: Env = None,
         __app: str = None,
         **attributes: type,
     ):
-        """
-        Args:
-            namespace:
-                prefix to use when getting settings from environment variables;
-                defaults to class namespace
-            __env:
-            __app: App name, primarily for django-related Config subclass usage.
-            **attributes: additional annotations to use
-        """
+        if attributes:
+            for key, annotation in attributes.items():
+                assert cls.is_setting(cls.as_setting_name(key)), (key, annotation)
+            return type(  # type: ignore
+                cls.__name__,
+                (cls, ),
+                {
+                    "__annotations__": attributes,
+                    "__doc__": {}
+                },
+            )(namespace, __env, __app)
+        return super().__new__(cls)
 
-        self.__attributes: dict = {}
-        self.__defaults = {}
+    def __init__(  # pylint: disable = unused-argument
+        self,
+        namespace: Optional[str] = None,
+        __env: Env = None,
+        __app: str = None,
+        **attributes: type,  # noqa: F841
+    ):
+        self.__attributes: Dict[str, SettingInfo] = {}
         self.__env = __env
         self.__app = __app
         self.__configured = False
@@ -347,65 +441,23 @@ class Config:
             raise ValueError(f"Invalid namespace {namespace!r}")
         self.__namespace = namespace
 
-        reserved = frozenset(k.upper() for k in dir(self) if self.is_setting(k.upper()))
-        for key, ann in attributes.items():
-            name = key.upper()
-            assert name not in reserved, (key, ann)
-            assert name not in self.__attributes, (key, ann)
-            assert self.is_setting(name), (key, ann)
-            assert callable(getattr(Env, ann.__name__)), (key, ann)
-            self.__attributes[name] = ann
-
-        for key, ann in self.__annotations__.items():
-            # should be ok from __init_subclass__
-            assert not _is_immutable(ann) and self.is_setting(self.as_setting_name(key))
-            if hasattr(self.__class__, key) and getattr(self.__class__, key) is None:
-                if isinstance(ann, type(Annotated[str, ""])):  # coverage: exclude
-                    ann = Annotated[(NotNull[ann.__origin__], *ann.__metadata__)]  # type: ignore
-                else:
-                    ann = NotNull[ann]
-            name = self.as_setting_name(key)
-            if isinstance(getattr(self.__class__, as_property_name(name), None), property):
-                assert not (isinstance(ann, type(Annotated[str, ""])) or get_origin(NotNull[ann]) is Union), (key, ann)
-                self.__attributes[name] = Annotated[NotNull[ann], empty]
-                continue
-            assert get_origin(ann) is not Union, (key, ann)
-            if name not in self.__attributes:
-                _ann = ann
-                if isinstance(ann, type(Annotated[str, ""])):
-                    _ann = ann.__origin__
-                    assert get_origin(_ann) is not Union, (key, ann)
-                _ann = get_origin(_ann) or _ann
-                assert hasattr(_ann, "__name__"), (_ann, ann)
-                env_call = getattr(Env, _ann.__name__, None)
-                process_name = f"_process_{as_property_name(name)}"
-                process_call = getattr(self, process_name, None)
-                error_text = f"missing {process_name} for {key}: {inspect.formatannotation(ann)}"
-                if not (callable(env_call) or callable(process_call)):
-                    assert isinstance(_ann, type) and issubclass(_ann, Config), error_text
-                self.__attributes[name] = ann
-            received = self.__attributes[name]
-            assert ann is received, f"`{name}` should be of type {ann.__name__}, got {received.__name__}"
-            if hasattr(self.__class__, key):
-                self.__defaults[name] = getattr(self.__class__, key)
+        for key, setting_info in self.__info__.items():
+            assert self.is_setting(setting_info.name)  # should be ok from __init_subclass__
+            assert not setting_info.is_immutable  # should be ok from __init_subclass__
+            assert setting_info.name not in self.__attributes, (key, setting_info)
+            self.__attributes[setting_info.name] = setting_info
 
     def _get(self, name):
         assert self.is_setting(name), name
-        ann = self.__attributes[name]
-        if isinstance(getattr(self.__class__, as_property_name(name), None), property):
+        if self.__attributes[name].is_property:
             return super().__getattribute__(as_property_name(name))
-        if isinstance(ann, type) and issubclass(ann, Config):
-            return ann(None, self.env).populate_from_env(parent_namespace=self.__namespace)
-        env_name = name
-        if (
-            isinstance(ann, type(Annotated[str, ""])) and len(getattr(ann, "__metadata__")) == 1 and
-            isinstance(getattr(ann, "__metadata__")[0], str)
-        ):
-            env_name = getattr(ann, "__metadata__")[0]
-        env_name = "_".join(filter(None, (self.__namespace, env_name)))
+        if self.__attributes[name].is_config:
+            obj = self.__attributes[name].attr_ann(None, self.env)
+            return obj.populate_from_env(parent_namespace=self.__namespace)
+        env_name = "_".join(filter(None, (self.__namespace, self.__attributes[name].env_name)))
         (k, v), = self.env.get_by_annotations(
             self,
-            key_to_name=lambda key: "_".join(filter(None, (self.__namespace, key))),
+            key_to_name=lambda key: "_".join(filter(None, (self.__namespace, self.__info__[key].env_name))),
             name_is_setting=lambda name_: name_ == env_name,
         )
         assert name == k, (name, k, v)
@@ -424,62 +476,20 @@ class Config:
             setattr(self, setting_name, self._get(setting_name))
         return super().__getattribute__(setting_name)
 
-    def __setattr__(self, __name: str, __value: Any) -> None:
-        env_name = self.as_setting_name(__name)
+    def __setattr__(self, __name: str, __value: Any, *, no_validation=False) -> None:
+        name = self.as_setting_name(__name)
 
-        if not __name.startswith("_") and self.is_setting(env_name):
-            assert env_name in self.__attributes, env_name
-            ann = self.__attributes[env_name]
-            if (
-                isinstance(ann, type(Annotated[str, ""])) and len(ann.__metadata__) == 1 and
-                (isinstance(ann.__metadata__[0], str) or ann.__metadata__[0] is empty)
-            ):
-                ann = ann.__origin__
-
-            received = type(__value)
-            error_message = f"`{env_name}` should be of type {ann}, got {received.__name__} {__value!r}"
-
-            origin = get_origin(ann)
-            multi = False
-            if origin in (list, dict, tuple):
-                args = ann.__args__
-                if origin is tuple and args == ((), ):
-                    args = ()
-                elif origin is tuple and len(args) == 2 and args[-1] is ...:
-                    args = (args[0], )
-                    multi = True
-                assert all(isinstance(arg, type) for arg in args), ann
-            else:
-                assert origin is None and isinstance(ann, type), ann
-                args = empty
-
-            if __value is None:
-                info_ann = self.__info__[env_name].annotation
-                assert Optional[info_ann] == info_ann, error_message
-            elif origin is None:
-                assert received is ann, error_message
-            else:
-                assert received is origin, error_message
-                if origin is tuple and not multi:
-                    assert len(args) == len(__value), error_message
-                    assert all(isinstance(x, t) for x, t in zip(__value, args)), error_message
-                elif origin is list or origin is tuple:
-                    item_ann, = args
-                    assert all(isinstance(x, item_ann) for x in __value), error_message
-                else:
-                    assert origin is dict
-                    kat, vat = args
-                    assert all(isinstance(k, kat) and isinstance(v, vat) for k, v in __value.items()), error_message
-
-            __name = env_name
+        if not __name.startswith("_") and self.is_setting(name):
+            assert no_validation or validate_setting_value(
+                name=name,
+                value=__value,
+                annotation=self.__attributes[name].attr_ann,
+                allow_null=self.__attributes[name].allow_null,
+            )
+            __name = name
 
         super().__setattr__(__name, __value)
 
-    def get_default(self, key, default):
-        """Fetches default value. Primarily for hiddenv.environment.Env.get_by_annotations."""
-
-        return self.__defaults.get(key, default)
-
     def load(self, __call: KwargsCallable[_T], **overrides) -> _T:
         """Returns result of given kwargs callable.
 
@@ -488,7 +498,13 @@ class Config:
 
         return load(__call, partial(getattr, self), **overrides)
 
-    def populate_from_env(self, env: Env = None, parent_namespace: str = None, *, target: Any = empty):
+    def populate_from_env(
+        self,
+        env: Env = None,
+        parent_namespace: str = None,
+        *,
+        target: Any = None,
+    ):
         """Populates settings to target (default self) from environment and returns the target."""
 
         if env is not None:  # coverage: exclude
@@ -506,19 +522,18 @@ class Config:
             for k, v in self.__annotations__.items()
             if isinstance(v, type) and issubclass(v, Config)
         }
-        if target is empty:
-            target = self
+        set_attribute = partial(self.__setattr__, no_validation=True) if target is None else partial(setattr, target)
         for k, v in self.env.get_by_annotations(
             self,
             key_to_name=lambda key: "_".join(filter(None, (namespace, self.as_setting_name(key)))),
             name_is_setting=lambda name: bool(self.is_setting(name)),
             custom_processing=custom_processing,
         ):
-            setattr(target, k, v)
-        return target
+            set_attribute(k, v)
+        return self if target is None else target
 
     @classmethod
-    def get_dotenv(cls):
+    def get_dotenv(cls, exclude=()):
         """Creates and returns `cls.dotenv` using `cls.__info__`."""
 
         data: Dict[Type[Config], List[SettingInfo]] = {}
@@ -529,12 +544,13 @@ class Config:
                 data[setting_info.parent] = []
             if isinstance(setting_info.annotation, type) and issubclass(setting_info.annotation, Config):
                 for sub_si in setting_info.annotation.__info__.values():
-                    if sub_si.is_property:
+                    if sub_si.is_property or sub_si.name in exclude:
                         continue
                     assert not (isinstance(sub_si.annotation, type) and issubclass(sub_si.annotation, Config))
                     data[setting_info.parent].append(sub_si)
             else:
-                data[setting_info.parent].append(setting_info)
+                if setting_info.name not in exclude:
+                    data[setting_info.parent].append(setting_info)
         full_text = ""
         for parent, info_list in data.items():
             parsed = docs.parse(parent.__doc__ or "")
@@ -563,41 +579,45 @@ class Config:
     @classmethod
     def _get_reserved(cls) -> Dict[str, _Reserved]:
         reserved = {}
-        for source in map(cls._Reserved, (*cls.__bases__[::-1], cls)):
+        for source in map(_Reserved, (*cls.__bases__[::-1], cls)):
             reserved.update({k: source for k in source.get_reserved(cls.as_setting_name, cls.is_setting)})
         return reserved
 
     @classmethod
-    def _get_reviewable(cls, reserved: Dict[str, _Reserved]) -> Tuple[_Annotation, ...]:
-        reviewable: Dict[str, _Annotation] = {}
+    def _get_reviewable(cls, reserved: Dict[str, _Reserved], exclude: Collection[str]) -> Tuple[_AnnotationInfo, ...]:
+        reviewable: Dict[str, _AnnotationInfo] = {}
 
         for base in (b for b in (*cls.__bases__[::-1], cls) if issubclass(b, Config)):
 
             for key, annotation in base.__annotations__.items():
 
-                new = cls._Annotation(
+                if key in exclude:
+                    continue
+
+                new = _AnnotationInfo(
                     key=key,
                     annotation=annotation,
                     source=base,
-                    name=cls.as_setting_name(key),
-                    property_name=as_property_name(cls.as_setting_name(key)),
+                    owner=cls,
                 )
 
                 if cls.is_setting(new.name):  # check anything that is a setting
 
-                    if not new.immutable and new.name in reserved:
+                    if not new.is_immutable and new.name in reserved:
                         raise TypeError(f"{key} reserved by {reserved[new.name]}")
 
-                    if new.name in reviewable and (reviewable[new.name].immutable or new.immutable):
+                    if new.name in reviewable and (reviewable[new.name].is_immutable or new.is_immutable):
                         if reviewable[new.name].annotation != new.annotation:
                             raise TypeError(f"\nIncompatible annotations:\n    {reviewable[new.name]}\n    {new}")
 
                     reviewable[new.name] = new
 
-        return tuple(x for x in reviewable.values() if not x.immutable)
+        return tuple(x for x in reviewable.values() if not x.is_immutable)
 
     @classmethod
-    def _get_info(cls, reviewable: Tuple[_Annotation, ...]) -> Dict[str, SettingInfo]:
+    def _get_info(cls, reviewable: Tuple[_AnnotationInfo, ...]) -> Dict[str, SettingInfo]:
+        if "__info__" in cls.__dict__:  # coverage: exclude
+            raise TypeError("do not define autogenerated class attribute __info__")
 
         info: Dict[str, SettingInfo] = {}
         for base in (b for b in cls.__bases__[::-1] if issubclass(b, Config)):
@@ -622,9 +642,16 @@ class Config:
 
             description = descriptions.get(obj.key, "")
             parent = cls
+
+            default = cls.__dict__.get(obj.key, empty)
+            if default is not empty:
+                delattr(cls, obj.key)
+
             if obj.key in info:
                 if not description:
                     description = info[obj.key].description
+                if default is empty:
+                    default = info[obj.key].default
                 parent = info[obj.key].parent
 
             info[obj.key] = SettingInfo(
@@ -633,28 +660,28 @@ class Config:
                 parent=parent,
                 owner=cls,
                 description=description,
-                default=getattr(cls, obj.key, empty),
+                default=default,
             )
 
         return info
 
-    def __init_subclass__(cls, *, namespace: str = "", **kwargs):
-        if namespace or namespace is EMPTY_NAMESPACE:
+    def __init_subclass__(cls, *, namespace: str = "", exclude: Tuple[str, ...] = (), **kwargs):
+        if namespace is EMPTY_NAMESPACE:
+            cls.namespace = namespace
+        elif namespace:
+            if not cls.is_setting(namespace):
+                raise ValueError(f"Invalid namespace {namespace!r}")
             cls.namespace = namespace
-        if cls.namespace:
-            assert cls.is_setting(cls.namespace), cls.namespace
 
         super().__init_subclass__(**kwargs)  # type: ignore
 
-        if "__info__" in cls.__dict__:  # coverage: exclude
-            raise TypeError("do not define autogenerated class attribute __info__")
         if "__doc__" not in cls.__dict__:  # coverage: exclude
             cls.__doc__ = ""
         if "__annotations__" not in cls.__dict__:
             cls.__annotations__ = {}
 
         reserved = cls._get_reserved()
-        reviewable = cls._get_reviewable(reserved)
+        reviewable = cls._get_reviewable(reserved, exclude)
         info = cls._get_info(reviewable)
 
         cls.__reserved__ = frozenset(reserved)
@@ -662,8 +689,7 @@ class Config:
         cls.__info__ = MappingProxyType(info)
         cls.get_dotenv()
 
-    _Reserved = _Reserved
-    _Annotation = _Annotation
 
+Config.__new__.__doc__ = Config.__init__.__doc__ = Config.__doc__
 
 _TConfig = TypeVar("_TConfig", bound=Type[Config])
